/*
 *  tcpbug.c  --  Accept a TCP/IP connection, and relay it to somewhere
 *		  else, while spying on the connection
 *
 *
 *  This program is in the public domain.  You may do anything
 *  you like with it.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkping University
 *		Sweden
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "ip/ip_misc.h"


#define TIMEOUT			10000	/* 10 seconds */


char		* progname;
int		  quiet_mode	= 0;


void
log(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    if (!quiet_mode)
	vfprintf(stderr, fmt, args);
    va_end(args);
}


void
fatal(char	* msg)
{
    extern char	* sys_errlist[];

    fprintf(stderr, "%s: %s: %s\n", progname, msg, sys_errlist[errno]);
    exit(1);
}



/*
 *  Relay all communication between the client and the server, and also
 *  copy everything to stdout.
 */
void
bug_me(fd_service, fd_client)
      int fd_service;
      int fd_client;
{
    fd_set readset;
    struct timeval wait;
    int nfds, len;
    char buffer[8192];
  
  
    /* Setup timeout structure */
    wait.tv_sec  = TIMEOUT / 1000;
    wait.tv_usec = (TIMEOUT * 1000) % 1000000;
  
    while (1)
    {
	/* Setup file descriptor set */
	FD_ZERO(&readset);
	FD_SET(fd_service, &readset);
	FD_SET(fd_client, &readset);
    
    
	nfds = select(FD_SETSIZE,
		      &readset,
		      (fd_set *) NULL,
		      (fd_set *) NULL,
		      &wait);
    
    
	/* error? */
	if (nfds < 0)
	{
	    if (errno == EINTR)
		continue;
	    perror("in.relayd: select error");
	    return;
	}
    
	/* timeout? */
	if (nfds == 0)
	    continue;
    
	if (FD_ISSET(fd_service, &readset))
	{
	    if ((len = read(fd_service, buffer, sizeof(buffer))) < 0)
	    {
		perror("in.relayd: remote service");
		return;
	    }

	    /* remote service close? */
	    if (len == 0)
	    {
		close(fd_client);
		return;
	    }
      
	    write(fd_client, buffer, len);
	    write(fileno (stdout), buffer, len);
	}
    
	if (FD_ISSET(fd_client, &readset))
	{
	    if ((len = read(fd_client, buffer, sizeof(buffer))) < 0)
	    {
		perror("in.relayd: remote client");
		return;
	    }
      
	    /* remote client close? */
	    if (len == 0)
	    {
		close(fd_service);
		return;
	    }
      
	    write(fd_service, buffer, len);
	    write(fileno (stdout), buffer, len);
	}

    }
}



void
usage ()
{
    fprintf(stderr, "usage: %s  my-port  server-host  server-port\n",
	    progname);
    fprintf(stderr,
	    "\t%s is a program that bugs a TCP/IP connection.  It waits\n"
	    "\tfor connections on a port that you specify, and then connects\n"
	    "\tto a server on a machine and a port you specify.  It then\n"
	    "\tprints out everything that is sent on the connection to\n"
	    "\tstdout.\n", progname
	);
    exit (1);
}


int
main (argc, argv)
      int argc;
      char **argv;
      
{
    struct sockaddr_in	  cliaddr;
    int			  cliaddrlen	= sizeof cliaddr;
    int			  listen_fd;
    int			  client_fd;
    int			  server_fd;


    progname = argv[0];

    if (argc != 4)
	usage();

    listen_fd = tcp_listen(NULL, argv[1]);
    if (listen_fd < 0)
	fatal("can't listen to port");
    client_fd = accept(listen_fd, (struct sockaddr *)&cliaddr, &cliaddrlen);
    if (client_fd < 0)
	fatal("accept failed");
    log("Connection from %s port %d\n",
	inet_ntoa(cliaddr.sin_addr), ntohs(cliaddr.sin_port));
    close(listen_fd);
    server_fd = tcp_open(argv[2], argv[3], NULL, NULL);
    if (server_fd < 0)
	fatal("can't connect to server");

    bug_me(server_fd, client_fd);

    return 0;
}
